function thetastore = get_gaussfitter(allspots,ROI,parameters)
% This m-file is for single emitter localization using a vectorial
% dipole PSF with a Gaussian PSF model.

% (C) Copyright 2018
% All rights reserved
% Department of Imaging Physics
% Faculty of Applied Sciences
% Delft University of Technology
% Delft, The Netherlands   

model = 'standard2D';
switch model
    case 'standard2D'
        numparams = 5;
    case 'astigmatic3D'
        numparams = 6;
    case 'diffractivecolor2D'
        numparams = 6;
    case 'diffractivecolor3D'
        numparams = 7;
end
recalc2d = 1;

[Nxpixels,Nypixels,~] = size(allspots);

% allspots = allspots((ceil(Nxpixels/2)-floor(ROI/2):ceil(Nxpixels/2)+floor(ROI/2)),(ceil(Nypixels/2)-floor(ROI/2):ceil(Nypixels/2)+floor(ROI/2)),:,:);
allspots = allspots((ceil(Nxpixels/2)-floor(ROI/2):ceil(Nxpixels/2)+floor(ROI/2))+round(parameters.xemit/80),(ceil(Nypixels/2)-floor(ROI/2):ceil(Nypixels/2)+floor(ROI/2)),:);

[Nxpixels,Nypixels,~] = size(allspots);

pixelsize = parameters.pixelsize*parameters.NA/parameters.lambda;

Duxstr = 0;%parameters.yemit/parameters.pixelsize;
Duystr = 0;%parameters.xemit/parameters.pixelsize;

% pupil and image size and coordinate sampling
ImageSizex = Nxpixels*pixelsize/2;
ImageSizey = Nypixels*pixelsize/2;
ximag = -ImageSizex+pixelsize/2:pixelsize:ImageSizex;
yimag = -ImageSizey+pixelsize/2:pixelsize:ImageSizey;
[XImage,YImage] = meshgrid(ximag,yimag);

% readout noise settings, and the variance value varfit used in the spot
% fitting, a value varfit >> 1 is the LS-limit, a value = 0 is the
% shot noise MLE-limit.
readnoisemean = 0.0;
readnoisestd = 0.0;
readnoisevariance = readnoisestd^2;
varfit = readnoisevariance;

% optical parameter settings: polar and azimuthal angles, NA, refractive
% index medium, cover slip, immersion medium, wavelength (nm), magnification.
settings.model = model;
settings.pola = 0;
settings.azim = 0.0;
settings.NA = parameters.NA;
settings.refmed = parameters.refmed;
settings.refcov = parameters.refcov;
settings.refimm = parameters.refimm;
settings.lambda = parameters.lambda;
settings.pixelsize = pixelsize;
settings.Nxpixels = Nxpixels;
settings.Nypixels = Nypixels;

switch model
    case 'standard2D'
        phasedepth = 0.0;
        pitch = 0.75;
        scurvelength = 0;
        Fmax = 0.0;
    case 'astigmatic3D'
        phasedepth = 0.0;
        pitch = 0.75;
        scurvelength = 500;
        Fmax = 0.90;
    case 'diffractivecolor2D'
        phasedepth = settings.lambda/3;
        pitch = 0.75;
        scurvelength = 0;
        Fmax = 0.0;
    case 'diffractivecolor3D'
        phasedepth = settings.lambda/3;
        pitch = 0.50;
        scurvelength = 300;
        Fmax = 0.60;
end
settings.scurvelength = scurvelength;
settings.phasedepth = phasedepth;
settings.pitch = pitch;

%%

Nsteps = 1;

% allparsmean = zeros(Nsteps,numparams);
% allparsstd = zeros(Nsteps,numparams);
% allCRLBmean = zeros(Nsteps,numparams,numparams);
% allspots = zeros(Nypixels,Nxpixels,Ncfg);
switch model
    case {'astigmatic3D','diffractivecolor3D'}
        allzestmean = zeros(Nsteps,1);
        allzeststd = zeros(Nsteps,1);
        allCRLBzmean = zeros(Nsteps,1);
end

% parameter settings
paramaters.numparams = numparams;
paramaters.model = model;
paramaters.Nxpixels = Nxpixels;
paramaters.Nypixels = Nypixels;
paramaters.pixelsize = pixelsize;
paramaters.Nitermax = 75;
paramaters.varfit = varfit;
paramaters.convpostolam = settings.pitch*settings.lambda;
paramaters.phasedepth = phasedepth;
paramaters.Fmax = Fmax;

% estimation of initial values for the fit parameters
% tic
thetastore = initialvalues(paramaters,allspots);
% toc

% Use initial fits as starting point in MLE.
% tic
if recalc2d
    % MLE fit routine.
    [thetastore, sumdthetastore, meritstore, stdstore] = localization(paramaters,thetastore,allspots);
    % calculation of log-likelihood offset.
    meritoffset = meritoffsetcalc(allspots,varfit);
    for jiter = 1:paramaters.Nitermax
        meritstore(:,jiter) = meritstore(:,jiter)+meritoffset;
    end
end
% toc

% calculation of Fisher-matrix and CRLB at found parameter-values
CRLBstore = FisherCRLB(paramaters,thetastore,readnoisevariance);

% calibration of the axial coordinate
switch model
    case 'astigmatic3D'
        ztrue = Duzstr*settings.lambda;
        Fest = thetastore(:,6);
        Fest = min(Fest,Fmax);
        Fest = max(Fest,-Fmax);
        sigmaz = thetastore(:,3);
        [Fmax_fit,zmax_fit,zest,sigma0] = zcalibrate(Fest,sigmaz,ztrue);
        scurvelength_fit = 2*Fmax_fit*zmax_fit;
        dof_fit = zmax_fit*sqrt(1-Fmax_fit^2);
        dzdF = zmax_fit*Fmax./sqrt(Fmax^2-Fest.^2)./(Fmax+sqrt(Fmax^2-Fest.^2));
        CRLBzstore = dzdF.^2.*squeeze(CRLBstore(6,6,:));
    case 'diffractivecolor3D'
        ztrue = Duzstr*settings.lambda;
        Fest = thetastore(:,7);
        Fest = min(Fest,Fmax);
        Fest = max(Fest,-Fmax);
        sigmaz = thetastore(:,3);
        [Fmax_fit,zmax_fit,zest,sigma0] = zcalibrate(Fest,sigmaz,ztrue);
        scurvelength_fit = 2*Fmax_fit*zmax_fit;
        dof_fit = zmax_fit*sqrt(1-Fmax_fit^2);
        zmax = 2*Fmax/(mean(Fest.*ztrue)/mean(ztrue.^2));
        dzdF = zmax*Fmax./sqrt(Fmax^2-Fest.^2)./(Fmax+sqrt(Fmax^2-Fest.^2));
        CRLBzstore = dzdF.^2.*squeeze(CRLBstore(7,7,:));
end
%%
% Analysis of the data.
%

%     outlier removal based on statistics of maximum log-likelihood, on spot
%     size, and on found position
outliers = (abs(thetastore(:,1))>2*pixelsize)|...
    (abs(thetastore(:,2))>2*pixelsize);

switch model
    case 'standard2D'
        outliers = outliers|(thetastore(:,3)>2*pixelsize);
    case {'astigmatic3D','diffractivecolor3D'}
        outliers = outliers|(thetastore(:,3)>3*pixelsize);
    case 'diffractivecolor2D'
        outliers = outliers|(thetastore(:,3)>2*pixelsize);
end

outliers = 0;

if (recalc2d)
    meritfinal = squeeze(meritstore(:,end));
    meanmeritfinal = mean(meritfinal);
    stdmeritfinal = std(meritfinal);
    outliers = outliers|(meritfinal<meanmeritfinal-3*stdmeritfinal);
end

% calculation of the mean and standard deviation of the results.
thetastore(:,1) = thetastore(:,1)-Duxstr;
thetastore(:,2) = thetastore(:,2)-Duystr;
switch model
    case {'astigmatic3D','diffractivecolor3D'}
        zest = zest-ztrue;
        zestmean = mean(zest(~outliers));
        zeststd = sqrt(var(zest(~outliers)));
        sigma0mean = mean(sigma0(~outliers));
        sigma0std = sqrt(var(sigma0(~outliers)));
end
thetamean = mean(thetastore(~outliers,:),1);
thetavar = var(thetastore(~outliers,:),1);
CRLBmean = mean(CRLBstore(:,:,~outliers),3);
thetastd = sqrt(thetavar);

% store mean and std in array for photon count runs
allparsmean = thetamean;
allparsstd = thetastd;
allCRLBmean = CRLBmean;

switch model
    case {'astigmatic3D','diffractivecolor3D'}
        allzestmean = zestmean;
        allzeststd = zeststd;
        allCRLBzmean = mean(CRLBzstore(~outliers));
end

end

